package com.ann.exceldemo.web.common;

import com.ann.exceldemo.web.GsonAdapter.DateTimeTypeAdapter;
import com.ann.exceldemo.web.GsonAdapter.JsonIntegerDeserializer;
import com.ann.exceldemo.web.GsonAdapter.MultiNameReflectiveTypeAdapterFactory;
import com.google.gson.*;
import com.google.gson.internal.ConstructorConstructor;
import com.google.gson.internal.Excluder;
import com.google.gson.internal.bind.*;
import com.google.gson.reflect.TypeToken;
import com.google.gson.stream.JsonReader;
import com.google.gson.stream.JsonToken;
import com.google.gson.stream.JsonWriter;
import org.apache.commons.lang3.StringUtils;

import java.io.IOException;
import java.lang.reflect.Type;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.*;
import java.util.concurrent.atomic.AtomicLong;
import java.util.concurrent.atomic.AtomicLongArray;

/**
 * @author taohongsheng
 * @date 2020/6/24
 * @description
 */
public class GsonUtils {

    /**
     * FORMATTER_L: 长日期格式化:yyyy-MM-dd HH:mm:ss
     */
    private static final String FORMATTER_L = "yyyy-MM-dd HH:mm:ss";
    /**
     * gson的一些默认配置
     */
    private static final boolean DEFAULT_COMPLEX_MAP_KEYS = false;
    private static final boolean DEFAULT_SPECIALIZE_FLOAT_VALUES = false;

    /**
     * 默认的 gsonBuilder 对象
     *
     * @return Gson对象
     */
    public static GsonBuilder defaultGsonBuilder() {
        return builderDefaultSetting(null);
    }

    /**
     * 创建默认Gson对象
     * 默认支持：
     * 1.自定义日期转换
     * 2.支持标准的boolean类型转换为int  true=1 false=0
     *
     * @param gsonBuilder 已经创建的builder 对象
     * @return GsonBuilder对象
     */
    private static GsonBuilder builderDefaultSetting(GsonBuilder gsonBuilder) {
        if (gsonBuilder == null) {
            gsonBuilder = new GsonBuilder();
        }
        return gsonBuilder.setDateFormat(FORMATTER_L)
                .registerTypeAdapter(Date.class, new DateTimeTypeAdapter())
                .registerTypeAdapter(Integer.class, new JsonIntegerDeserializer())
                .registerTypeAdapter(int.class, new JsonIntegerDeserializer());
    }

    /**
     * 默认增强 gsonBuilder 对象
     *
     * @return GsonBuilder对象
     */
    public static GsonBuilder enhanceGsonBuilder() {
        return enhanceGsonBuilder(Excluder.DEFAULT, FieldNamingPolicy.LOWER_CASE_WITH_UNDERSCORES, Collections.<Type, InstanceCreator<?>>emptyMap());
    }

    /**
     * 可以指定对象名称转换格式
     *
     * @param policy 转换模式
     * @return GsonBuilder对象
     */
    public static GsonBuilder enhanceGsonBuilder(FieldNamingPolicy policy) {
        return enhanceGsonBuilder(Excluder.DEFAULT, policy, Collections.<Type, InstanceCreator<?>>emptyMap());
    }

    /**
     * 增强Gson实现
     * 除支持 @see #defaultGsonBuilder() 里的实现以外
     * 支持指定FieldNamingPolicy并兼容了相同名称的情况
     *
     * @return GsonBuilder对象
     */
    public static GsonBuilder enhanceGsonBuilder(Excluder excluder, FieldNamingPolicy policy, Map<Type, InstanceCreator<?>> instanceCreators) {
        //一些默认配置
        ConstructorConstructor constructorConstructor = new ConstructorConstructor(instanceCreators);
        List<TypeAdapterFactory> factories = new ArrayList<>();
        // type adapters for basic platform types
        factories.add(TypeAdapters.STRING_FACTORY);
        factories.add(TypeAdapters.INTEGER_FACTORY);
        factories.add(TypeAdapters.BOOLEAN_FACTORY);
        factories.add(TypeAdapters.BYTE_FACTORY);
        factories.add(TypeAdapters.SHORT_FACTORY);
        TypeAdapter<Number> longAdapter = longAdapter(LongSerializationPolicy.DEFAULT);
        factories.add(TypeAdapters.newFactory(long.class, Long.class, longAdapter));
        factories.add(TypeAdapters.newFactory(double.class, Double.class,
                doubleAdapter(DEFAULT_SPECIALIZE_FLOAT_VALUES)));
        factories.add(TypeAdapters.newFactory(float.class, Float.class,
                floatAdapter(DEFAULT_SPECIALIZE_FLOAT_VALUES)));
        factories.add(TypeAdapters.ATOMIC_INTEGER_FACTORY);
        factories.add(TypeAdapters.ATOMIC_BOOLEAN_FACTORY);
        factories.add(TypeAdapters.newFactory(AtomicLong.class, atomicLongAdapter(longAdapter)));
        factories.add(TypeAdapters.newFactory(AtomicLongArray.class, atomicLongArrayAdapter(longAdapter)));
        factories.add(TypeAdapters.ATOMIC_INTEGER_ARRAY_FACTORY);
        factories.add(TypeAdapters.CHARACTER_FACTORY);
        factories.add(TypeAdapters.STRING_BUILDER_FACTORY);
        factories.add(TypeAdapters.STRING_BUFFER_FACTORY);
        factories.add(TypeAdapters.newFactory(BigDecimal.class, TypeAdapters.BIG_DECIMAL));
        factories.add(TypeAdapters.newFactory(BigInteger.class, TypeAdapters.BIG_INTEGER));
        factories.add(TypeAdapters.URL_FACTORY);
        factories.add(TypeAdapters.URI_FACTORY);
        factories.add(TypeAdapters.UUID_FACTORY);
        factories.add(TypeAdapters.CURRENCY_FACTORY);
        factories.add(TypeAdapters.LOCALE_FACTORY);
        factories.add(TypeAdapters.INET_ADDRESS_FACTORY);
        factories.add(TypeAdapters.BIT_SET_FACTORY);
        factories.add(DateTypeAdapter.FACTORY);
        factories.add(TypeAdapters.CALENDAR_FACTORY);
        factories.add(ArrayTypeAdapter.FACTORY);
        factories.add(TypeAdapters.CLASS_FACTORY);

        factories.add(new CollectionTypeAdapterFactory(constructorConstructor));
        factories.add(new MapTypeAdapterFactory(constructorConstructor, DEFAULT_COMPLEX_MAP_KEYS));
        MultiNameReflectiveTypeAdapterFactory.JsonAdapterAnnotationTypeAdapterFactory jsonAdapterFactory = new MultiNameReflectiveTypeAdapterFactory.JsonAdapterAnnotationTypeAdapterFactory(constructorConstructor);
        factories.add(jsonAdapterFactory);
        factories.add(TypeAdapters.ENUM_FACTORY);
        //自己实现的，兼容解析gson时，不使用@SerializedName情况下，返回多个字段名称。见：MultiNameReflectiveTypeAdapterFactory.getFieldNames
        factories.add(new MultiNameReflectiveTypeAdapterFactory(constructorConstructor, policy, excluder, jsonAdapterFactory));
        //GsonBuilder 是后面覆盖前面的，所以要逆序下。详见 com.google.gson.GsonBuilder().create 方法
        Collections.reverse(factories);
        GsonBuilder gsonBuilder = new GsonBuilder();
        for (TypeAdapterFactory factory : factories) {
            gsonBuilder.registerTypeAdapterFactory(factory);
        }
        return builderDefaultSetting(gsonBuilder);
    }

    /**
     * jsonArray转java数组
     *
     * @param jsonElements
     * @param <T>
     * @return
     */
    public static <T> List<T> toJavaList(JsonArray jsonElements, Class<T> clazz) {
        List<T> result = new ArrayList<>();
        for (JsonElement element : jsonElements) {
            T t = defaultGsonBuilder().create().fromJson(element, clazz);
            result.add(t);
        }
        return result;
    }

    /**
     * JsonObject转对象
     *
     * @param jsonObject
     * @param clazz
     * @param <T>
     * @return
     */
    public static <T> T toJavaObject(JsonObject jsonObject, Class<T> clazz) {
        return enhanceGsonBuilder().create().fromJson(jsonObject, clazz);
    }

    /**
     * json串转 java对象
     *
     * @param jsonStr
     * @param clazz
     * @param <T>
     * @return
     */
    public static <T> T toJavaObject(String jsonStr, Class<T> clazz) {
        return enhanceGsonBuilder().create().fromJson(jsonStr, clazz);
    }

    /**
     * 下划线转驼峰
     *
     * @param jsonStr
     * @param clazz
     * @param fieldNamingPolicy
     * @param <T>
     * @return
     */
    public static <T> T toJavaObject(String jsonStr, Class<T> clazz, FieldNamingPolicy fieldNamingPolicy) {
        return defaultGsonBuilder().setFieldNamingStrategy(fieldNamingPolicy).create().fromJson(jsonStr, clazz);
    }


    /**
     * json串转 java对象
     *
     * @param jsonStr
     * @param type
     * @param <T>
     * @return
     */
    public static <T> T toJavaObject(String jsonStr, Type type) {
        return enhanceGsonBuilder().create().fromJson(jsonStr, type);
    }

    /**
     * 下划线转驼峰
     *
     * @param jsonStr
     * @param type
     * @param fieldNamingPolicy
     * @param <T>
     * @return
     */
    public static <T> T toJavaObject(String jsonStr, Type type, FieldNamingPolicy fieldNamingPolicy) {
        return enhanceGsonBuilder(fieldNamingPolicy).create().fromJson(jsonStr, type);
    }

    /**
     * json串转 jsonObject
     *
     * @param jsonStr
     * @return
     */
    public static JsonObject toJsonObject(String jsonStr) {
        if (StringUtils.isBlank(jsonStr)) {
            throw new IllegalArgumentException(jsonStr);
        }
        return JsonParser.parseString(jsonStr).getAsJsonObject();
    }

    /**
     * json串转 jsonArray
     *
     * @param jsonStr
     * @return
     */
    public static JsonArray toJsonArray(String jsonStr) {
        if (StringUtils.isBlank(jsonStr)) {
            throw new IllegalArgumentException(jsonStr);
        }
        return JsonParser.parseString(jsonStr).getAsJsonArray();
    }

    /**
     * 对象转JsonObject
     *
     * @param o
     * @return
     */
    public static JsonObject toJsonObject(Object o) {
        if (o == null) {
            return null;
        }
        if (o instanceof JsonObject) {
            return (JsonObject) o;
        }
        return new Gson().toJsonTree(o).getAsJsonObject();
    }

    /**
     * 对象转jsonArray
     *
     * @param o
     * @return
     */
    public static JsonArray toJsonArray(Object o) {
        if (o == null) {
            return null;
        }
        return new Gson().toJsonTree(o).getAsJsonArray();
    }

    /**
     * object转gson串
     *
     * @param ob
     * @return
     */
    public static String toJSONString(Object ob) {
        if (ob == null) {
            return null;
        }
        return new GsonBuilder().registerTypeAdapter(Date.class, new DateTimeTypeAdapter()).create().toJson(ob);
    }

    public static String getAsString(JsonObject jsonObject, String memberName) {
        JsonElement jsonElement = jsonObject.get(memberName);
        if (jsonElement == null) {
            return null;
        }
        if (jsonElement.isJsonNull()) {
            return null;
        }
        if (jsonElement.isJsonArray() || jsonElement.isJsonObject()) {
            return GsonUtils.toJSONString(jsonElement);
        }
        return jsonElement.getAsString();
    }

    public static Boolean getAsBoolean(JsonObject jsonObject, String memberName) {
        JsonElement jsonElement = jsonObject.get(memberName);
        if (jsonElement == null) {
            return null;
        }
        if (jsonElement.isJsonNull()) {
            return null;
        }
        return jsonObject.get(memberName).getAsBoolean();
    }

    public static Byte getAsByte(JsonObject jsonObject, String memberName) {
        JsonElement jsonElement = jsonObject.get(memberName);
        if (jsonElement == null) {
            return null;
        }
        if (jsonElement.isJsonNull()) {
            return null;
        }
        return jsonObject.get(memberName).getAsByte();
    }

    public static Short getAsShort(JsonObject jsonObject, String memberName) {
        JsonElement jsonElement = jsonObject.get(memberName);
        if (jsonElement == null) {
            return null;
        }
        if (jsonElement.isJsonNull()) {
            return null;
        }
        return jsonObject.get(memberName).getAsShort();
    }

    public static Integer getAsInteger(JsonObject jsonObject, String memberName) {
        JsonElement jsonElement = jsonObject.get(memberName);
        if (jsonElement == null) {
            return null;
        }
        if (jsonElement.isJsonNull()) {
            return null;
        }
        return jsonObject.get(memberName).getAsInt();
    }

    public static Long getAsLong(JsonObject jsonObject, String memberName) {
        JsonElement jsonElement = jsonObject.get(memberName);
        if (jsonElement == null) {
            return null;
        }
        if (jsonElement.isJsonNull()) {
            return null;
        }
        return jsonObject.get(memberName).getAsLong();
    }

    public static BigDecimal getAsBigDecimal(JsonObject jsonObject, String memberName) {
        JsonElement jsonElement = jsonObject.get(memberName);
        if (jsonElement == null) {
            return null;
        }
        if (jsonElement.isJsonNull()) {
            return null;
        }
        return jsonObject.get(memberName).getAsBigDecimal();
    }

    /**
     * 获取出JsonPrimitive中的值
     *
     * @param value
     * @return
     */
    public static Object getPrimitiveValue(JsonPrimitive value) {
        if (value == null) {
            return null;
        }
        if (value.isJsonNull()) {
            return null;
        }
        if (value.isString()) {
            return value.getAsString();
        }
        if (value.isBoolean()) {
            return value.getAsBoolean();
        }
        if (value.isNumber()) {
            return value.getAsNumber();
        }
        return null;
    }

    /**
     * java对象转map
     *
     * @param object
     * @return
     */
    public static Map<String, Object> toMap(Object object) {
        if (object == null) {
            return Collections.emptyMap();
        }
        Gson gson = new Gson();
        JsonElement jsonElement = gson.toJsonTree(object);
        return gson.fromJson(jsonElement, new TypeToken<Map<String, Object>>() {
        }.getType());
    }

    public static void setPrimitiveValue(JsonObject object, String name, Object value) {
        if (value == null) {
            object.add(name, JsonNull.INSTANCE);
        }
        if (value instanceof Number) {
            object.addProperty(name, (Number) value);
        }
        if (value instanceof String) {
            object.addProperty(name, (String) value);
        }
        if (value instanceof Boolean) {
            object.addProperty(name, (Boolean) value);
        }
        if (value instanceof Character) {
            object.addProperty(name, (Character) value);
        }
    }

    public static void addValue(JsonArray object, Object value) {
        if (value == null) {
            return;
        }
        if (value instanceof Number) {
            object.add((Number) value);
        }
        if (value instanceof String) {
            object.add((String) value);
        }
        if (value instanceof Boolean) {
            object.add((Boolean) value);
        }
        if (value instanceof Character) {
            object.add((Character) value);
        }
        if (value instanceof JsonElement) {
            object.add((JsonElement) value);
        }
    }

    private static TypeAdapter<Number> longAdapter(LongSerializationPolicy longSerializationPolicy) {
        if (longSerializationPolicy == LongSerializationPolicy.DEFAULT) {
            return TypeAdapters.LONG;
        }
        return new TypeAdapter<Number>() {
            @Override
            public Number read(JsonReader in) throws IOException {
                if (in.peek() == JsonToken.NULL) {
                    in.nextNull();
                    return null;
                }
                return in.nextLong();
            }

            @Override
            public void write(JsonWriter out, Number value) throws IOException {
                if (value == null) {
                    out.nullValue();
                    return;
                }
                out.value(value.toString());
            }
        };
    }

    private static TypeAdapter<AtomicLong> atomicLongAdapter(final TypeAdapter<Number> longAdapter) {
        return new TypeAdapter<AtomicLong>() {
            @Override
            public void write(JsonWriter out, AtomicLong value) throws IOException {
                longAdapter.write(out, value.get());
            }

            @Override
            public AtomicLong read(JsonReader in) throws IOException {
                Number value = longAdapter.read(in);
                return new AtomicLong(value.longValue());
            }
        }.nullSafe();
    }

    private static TypeAdapter<AtomicLongArray> atomicLongArrayAdapter(final TypeAdapter<Number> longAdapter) {
        return new TypeAdapter<AtomicLongArray>() {
            @Override
            public void write(JsonWriter out, AtomicLongArray value) throws IOException {
                out.beginArray();
                for (int i = 0, length = value.length(); i < length; i++) {
                    longAdapter.write(out, value.get(i));
                }
                out.endArray();
            }

            @Override
            public AtomicLongArray read(JsonReader in) throws IOException {
                List<Long> list = new ArrayList<Long>();
                in.beginArray();
                while (in.hasNext()) {
                    long value = longAdapter.read(in).longValue();
                    list.add(value);
                }
                in.endArray();
                int length = list.size();
                AtomicLongArray array = new AtomicLongArray(length);
                for (int i = 0; i < length; ++i) {
                    array.set(i, list.get(i));
                }
                return array;
            }
        }.nullSafe();
    }

    private static TypeAdapter<Number> doubleAdapter(boolean serializeSpecialFloatingPointValues) {
        if (serializeSpecialFloatingPointValues) {
            return TypeAdapters.DOUBLE;
        }
        return new TypeAdapter<Number>() {
            @Override
            public Double read(JsonReader in) throws IOException {
                if (in.peek() == JsonToken.NULL) {
                    in.nextNull();
                    return null;
                }
                return in.nextDouble();
            }

            @Override
            public void write(JsonWriter out, Number value) throws IOException {
                if (value == null) {
                    out.nullValue();
                    return;
                }
                double doubleValue = value.doubleValue();
                checkValidFloatingPoint(doubleValue);
                out.value(value);
            }
        };
    }

    private static TypeAdapter<Number> floatAdapter(boolean serializeSpecialFloatingPointValues) {
        if (serializeSpecialFloatingPointValues) {
            return TypeAdapters.FLOAT;
        }
        return new TypeAdapter<Number>() {
            @Override
            public Float read(JsonReader in) throws IOException {
                if (in.peek() == JsonToken.NULL) {
                    in.nextNull();
                    return null;
                }
                return (float) in.nextDouble();
            }

            @Override
            public void write(JsonWriter out, Number value) throws IOException {
                if (value == null) {
                    out.nullValue();
                    return;
                }
                float floatValue = value.floatValue();
                checkValidFloatingPoint(floatValue);
                out.value(value);
            }
        };
    }

    static void checkValidFloatingPoint(double value) {
        if (Double.isNaN(value) || Double.isInfinite(value)) {
            throw new IllegalArgumentException(value
                    + " is not a valid double value as per JSON specification. To override this"
                    + " behavior, use GsonBuilder.serializeSpecialFloatingPointValues() method.");
        }
    }
}
