package com.ann.exceldemo.web.common;

import com.google.gson.Gson;
import com.google.gson.JsonElement;
import com.google.gson.JsonPrimitive;
import org.apache.commons.beanutils.ConvertUtils;
import org.apache.commons.lang3.math.NumberUtils;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

/**
 * @Description: 数据类型转换工具类
 */
public class TypeConvertUtils {
    public static String DEFFAULT_DATE_FORMAT = "yyyy-MM-dd HH:mm:ss";


    public static <T> T cast(Object obj, Class<T> clazz) {
        if (obj == null) {
            return null;
        } else {
            return cast2(obj, clazz);
        }
    }

    public static <T> T cast(Object obj, Class<T> clazz, T def) {
        if (obj == null) {
            return def;
        } else {
            return cast2(obj, clazz);
        }
    }

    public static Long castToEpochSecond(Object obj) {
        Long longVal = cast(obj, Long.class);
        if (longVal == null) {
            return null;
        } else {
            return longVal / 1000;
        }
    }


    /**
     * 自定义的类型状态，支持一些复杂的默认转换行为。可执行扩展
     *
     * @param obj   需要转换的对象
     * @param clazz 需要转换变成的对象类型
     * @param <T>   返回的泛型
     * @return 返回转换完成的数据
     */
    @SuppressWarnings({"unchecked"})
    private static <T> T cast2(Object obj, Class<T> clazz) {
        if (obj == null) {
            if (clazz == int.class) {
                return (T) Integer.valueOf(0);
            } else if (clazz == long.class) {
                return (T) Long.valueOf(0);
            } else if (clazz == short.class) {
                return (T) Short.valueOf((short) 0);
            } else if (clazz == byte.class) {
                return (T) Byte.valueOf((byte) 0);
            } else if (clazz == float.class) {
                return (T) Float.valueOf(0);
            } else if (clazz == double.class) {
                return (T) Double.valueOf(0);
            } else if (clazz == boolean.class) {
                return (T) Boolean.FALSE;
            }
            return null;
        }

        if (clazz == null) {
            throw new IllegalArgumentException("clazz is null");
        }

        if (clazz == obj.getClass()) {
            return (T) obj;
        }
        if (obj instanceof JsonElement) {
            JsonElement jsonElement = (JsonElement) obj;
            if (jsonElement.isJsonPrimitive()) {
                JsonPrimitive primitive = (JsonPrimitive) jsonElement;
                if (primitive.isBoolean()) {
                    obj = primitive.getAsBoolean();
                } else if (primitive.isNumber()) {
                    obj = primitive.getAsNumber();
                } else if (primitive.isString()) {
                    obj = primitive.getAsString();
                }
            } else if (jsonElement.isJsonNull()) {
                return null;
            } else if (jsonElement.isJsonObject()) {
                return new Gson().fromJson(jsonElement, clazz);
            } else if (jsonElement.isJsonArray()) {
                return new Gson().fromJson(jsonElement, clazz);
            }
        }
        if (clazz == Date.class) {
            return (T) castToDate(obj);
        }
        //非数字字符串，尝试当日期转
        if (obj instanceof CharSequence && !NumberUtils.isNumber(obj.toString()) && Number.class.isAssignableFrom(clazz)) {
            long time = castToDate(obj).getTime();
            return (T) ConvertUtils.convert(time, clazz);
        }
        return (T) ConvertUtils.convert(obj, clazz);
    }

    public static Date castToDate(Object value) {
        if (value == null) {
            return null;
        } else if (value instanceof Date) {
            return (Date) value;
        } else if (value instanceof Calendar) {
            return ((Calendar) value).getTime();
        } else {
            if (value instanceof BigDecimal) {
                return new Date(longValue((BigDecimal) value));
            } else if (value instanceof Number) {
                return new Date(((Number) value).longValue());
            } else {
                if (value instanceof String) {
                    if (NumberUtils.isDigits((String) value)) {
                        return new Date(NumberUtils.toLong((String) value));
                    }
                    return parseDate((String) value);
                }
            }
        }
        throw new IllegalStateException("can not cast to Date, value : " + value);
    }

    public static Date parseDate(String strVal) {
        String format;
        if (NumberUtils.isNumber(strVal)) {
            //10位的是秒，13位是毫秒
            long time = strVal.length() < 13 ? NumberUtils.toLong(strVal) * 1000 : NumberUtils.toLong(strVal);
            return new Date(time);
        }
        if (strVal.length() == DEFFAULT_DATE_FORMAT.length() || (strVal.length() == 22 && strVal.charAt(10) != 'T')) {
            format = DEFFAULT_DATE_FORMAT;
        } else if (strVal.length() == 10) {
            format = "yyyy-MM-dd";
        } else if (strVal.length() == "yyyy-MM-dd HH:mm:ss".length()) {
            format = "yyyy-MM-dd HH:mm:ss";
        } else if (strVal.length() == 29 && strVal.charAt(26) == ':' && strVal.charAt(28) == '0') {
            format = "yyyy-MM-dd'T'HH:mm:ss.SSSXXX";
        } else if (strVal.length() == 23 && strVal.charAt(19) == ',') {
            format = "yyyy-MM-dd HH:mm:ss,SSS";
        } else if ((strVal.length() >= 21 && strVal.length() <= 23) && strVal.charAt(19) == '.' && strVal.charAt(10) == 'T') {
            format = "yyyy-MM-dd'T'HH:mm:ss.SSS";
        } else if (strVal.length() == 23 && strVal.charAt(19) == ':') {
            format = "yyyy-MM-dd HH:mm:ss:SSS";
        } else {
            format = "yyyy-MM-dd HH:mm:ss.SSS";
        }
        SimpleDateFormat dateFormat = new SimpleDateFormat(format);
        try {
            return dateFormat.parse(strVal);
        } catch (Exception e) {
            throw new IllegalStateException("can not cast to Date, value : " + strVal);
        }
    }

    public static void main(String[] args) {
        String date = "2020-09-01T12:00:02.4";
        Date date1 = parseDate(date);
        System.out.println(date1.getTime());
    }

    private static long longValue(BigDecimal decimal) {
        if (decimal == null) {
            return 0L;
        } else {
            int scale = decimal.scale();
            return scale >= -100 && scale <= 100 ? decimal.longValue() : decimal.longValueExact();
        }
    }
}
