package com.ann.exceldemo.web.json;

import com.ann.exceldemo.web.common.MyException;
import com.ann.exceldemo.web.constant.KEY_BIZ;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.collect.Maps;
import net.sf.json.JSONObject;
import net.sf.json.JsonConfig;
import net.sf.json.processors.PropertyNameProcessor;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.lang.reflect.Field;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.util.Map;
import java.util.concurrent.ConcurrentMap;

/**
 * JSON转换工具类
 */
public class JSONUtil {

    private static ObjectMapper mapper = new SimpleObjectMapper();
    private static final Logger logger = LoggerFactory.getLogger(JSONUtil.class);

    /**
     * 转换成JSON字符串
     *
     * @param obj
     * @return
     */
    public static String encode(Object obj) {
        try {
            return mapper.writeValueAsString(obj);
        } catch (Exception e) {
            logger.error("Handle exception:{}", e.getMessage(), e);
            throw new MyException(KEY_BIZ.JSON_PARSE_FAIL);
        }
    }

    private static class FieldNameCache {
        private static final ConcurrentMap<Class, Map<String, String>> fieldNameCache = Maps.newConcurrentMap();

        public static Map<String, String> getFieldNameMap(Class target) {
            Map<String, String> fieldNameMap = fieldNameCache.computeIfAbsent(target, (cls) -> initFieldNameMap(cls));
            return fieldNameMap;
        }

        public static Map<String, String> initFieldNameMap(Class target) {
            Map<String, String> fieldNameMap = Maps.newHashMap();
            Field[] fields = target.getDeclaredFields();
            for (Field field : fields) {
                JsonProperty jsonProperty = field.getAnnotation(JsonProperty.class);
                if (jsonProperty == null) {
                    continue;
                }
                String name = jsonProperty.value();
                if (StringUtils.isNotEmpty(name)) {
                    fieldNameMap.put(name, field.getName());
                    fieldNameMap.put(field.getName(), name);
                }
            }
            return fieldNameMap;
        }
    }

    private static PropertyNameProcessor JSON_PROPERTY_NAME_PROCESSOR = new PropertyNameProcessor() {
        private final ConcurrentMap<Class, Map<String, String>> fieldNameCache = Maps.newConcurrentMap();

        @Override
        public String processPropertyName(Class target, String fieldName) {
            Map<String, String> fieldNameMap = FieldNameCache.getFieldNameMap(target);
            String field = fieldNameMap.get(fieldName);
            if (field == null) {
                return fieldName;
            }
            return field;
        }

    };

    /**
     * 转换成JSONObject
     *
     * @param obj
     * @return
     */
    public static JSONObject encodeToJsonlibObject(Object obj) {
        if (obj == null) {
            return null;
        }
        JsonConfig jsonConfig = new JsonConfig();
        jsonConfig.registerJsonPropertyNameProcessor(obj.getClass(), JSON_PROPERTY_NAME_PROCESSOR);
        try {
            return JSONObject.fromObject(obj, jsonConfig);
        } catch (Exception e) {
            logger.error("Handle exception:{}", e.getMessage(), e);
            throw new MyException(KEY_BIZ.JSON_PARSE_FAIL);
        }
    }

    /**
     * 解析JSON字符串为指定类型对象
     *
     * @param json
     * @param clazz
     * @param <T>
     * @return
     */
    public static <T> T decode(String json, Class<T> clazz) {
        try {
            return mapper.readValue(json, clazz);
        } catch (Exception e) {
            logger.error("Handle exception:{}", e.getMessage(), e);
            throw new MyException(KEY_BIZ.JSON_PARSE_FAIL);
        }
    }

    private static PropertyNameProcessor JAVA_PROPERTY_NAME_PROCESSOR = new PropertyNameProcessor() {
        private final ConcurrentMap<Class, Map<String, String>> fieldNameCache = Maps.newConcurrentMap();

        @Override
        public String processPropertyName(Class target, String fieldName) {
            Map<String, String> fieldNameMap = FieldNameCache.getFieldNameMap(target);
            String field = fieldNameMap.get(fieldName);
            if (field == null) {
                return fieldName;
            }
            return field;
        }

    };

    /**
     * 解析JSONObject为指定类型对象
     *
     * @param jsonObject
     * @param clazz
     * @param <T>
     * @return
     */
    public static <T> T decodeJsonlibObject(JSONObject jsonObject, Class<T> clazz) {
        if (jsonObject == null) {
            return null;
        }
        JsonConfig jsonConfig = new JsonConfig();
        jsonConfig.setRootClass(clazz);
        jsonConfig.registerJavaPropertyNameProcessor(clazz, JAVA_PROPERTY_NAME_PROCESSOR);
        try {
            return (T) JSONObject.toBean(jsonObject, jsonConfig);
        } catch (Exception e) {
            logger.error("Handle exception:{}", e.getMessage(), e);
            throw new MyException(KEY_BIZ.JSON_PARSE_FAIL);
        }
    }

    /**
     * 解析JSON串为指定类型，支持参数化类型
     *
     * @param json
     * @param typeReference
     * @param <T>
     * @return
     */
    public static <T> T decode(String json, TypeReference<T> typeReference) {
        try {
            return mapper.readValue(json, mapper.getTypeFactory().constructType(typeReference.getType()));
        } catch (Exception e) {
            logger.error("Handle exception:{}", e.getMessage(), e);
            throw new MyException(KEY_BIZ.JSON_PARSE_FAIL);
        }
    }

    /**
     * 类型引用
     *
     * @param <T>
     */
    public static abstract class TypeReference<T> {
        protected final Type actualType;

        protected TypeReference() {
            Type superClass = getClass().getGenericSuperclass();
            if (superClass instanceof Class<?>) {
                throw new MyException(KEY_BIZ.WITHOUT_ACTUAL_TYPE);
            }
            actualType = ((ParameterizedType) superClass).getActualTypeArguments()[0];
        }

        public Type getType() {
            return actualType;
        }
    }

}